@@ -31,6 +31,7 @@ def train(params, train_set, num_boost_round=100,
     ----------
     params : dict
         Parameters for training.
+        See https://github.com/fabsig/GPBoost/blob/master/docs/Main_parameters.rst#tuning-parameters--hyperparameters-for-the-tree-boosting-part
     train_set : Dataset
         Data to be trained on.
     num_boost_round : int, optional (default=100)
@@ -51,7 +52,8 @@ def train(params, train_set, num_boost_round=100,
     valid_names : list of strings or None, optional (default=None)
         Names of ``valid_sets``.
     fobj : callable or None, optional (default=None)
-        Customized objective function.
+        Customized objective function. Only for independent boosting.
+        The GPBoost algorithm currently does not support this.
         Should accept two parameters: preds, train_data,
         and return (grad, hess).
 
@@ -156,7 +158,7 @@ def train(params, train_set, num_boost_round=100,
     -------
     >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
     >>> data_train = gpb.Dataset(X, y)
-    >>> params = {'objective': 'regression_l2', 'verbose': 0}
+    >>> params = {'learning_rate': 0.01, 'max_depth': 3, 'num_leaves': 2**10, 'verbose': 0}
     >>> bst = gpb.train(params=params, train_set=data_train,  gp_model=gp_model,
     >>>                 num_boost_round=100)
 
@@ -485,7 +487,8 @@ def _make_n_folds(full_data, folds, nfold, params, seed, gp_model=None, use_gp_m
                                                    cluster_ids_pred=cluster_ids_pred,
                                                    vecchia_pred_type=gp_model.vecchia_pred_type,
                                                    num_neighbors_pred=gp_model.num_neighbors_pred,
-                                                   cg_delta_conv_pred=gp_model.cg_delta_conv_pred)
+                                                   cg_delta_conv_pred=gp_model.cg_delta_conv_pred,
+                                                   nsim_var_pred=gp_model.nsim_var_pred)
             cvbooster = Booster(params=tparam, train_set=train_set, gp_model=gp_model_train)
             gp_model._set_likelihood(
                 gp_model_train._get_likelihood_name())  # potentially change likelihood in case this was done in the booster to reflect implied changes in the default optimizer for different likelihoods
@@ -519,18 +522,19 @@ def cv(params, train_set, num_boost_round=100,
        gp_model=None, use_gp_model_for_validation=True,
        fit_GP_cov_pars_OOS=False, train_gp_model_cov_pars=True,
        folds=None, nfold=5, stratified=False, shuffle=True,
-       metrics=None, fobj=None, feval=None, init_model=None,
+       metric=None, fobj=None, feval=None, init_model=None,
        feature_name='auto', categorical_feature='auto',
        early_stopping_rounds=None, fpreproc=None,
        verbose_eval=None, show_stdv=False, seed=0,
        callbacks=None, eval_train_metric=False,
-       return_cvbooster=False):
+       return_cvbooster=False, metrics=None):
     """Perform cross-validation for choosing number of boosting iterations.
 
     Parameters
     ----------
     params : dict
         Parameters for Booster.
+        See https://github.com/fabsig/GPBoost/blob/master/docs/Main_parameters.rst#tuning-parameters--hyperparameters-for-the-tree-boosting-part
     train_set : Dataset
         Data to be trained on.
     num_boost_round : int, optional (default=100)
@@ -562,11 +566,12 @@ def cv(params, train_set, num_boost_round=100,
         Whether to perform stratified sampling.
     shuffle : bool, optional (default=True)
         Whether to shuffle before splitting data.
-    metrics : string, list of strings or None, optional (default=None)
-        Evaluation metrics to be monitored while CV.
+    metric : string, list of strings or None, optional (default=None)
+        Evaluation metric to be monitored when doing CV.
         If not None, the metric in ``params`` will be overridden.
     fobj : callable or None, optional (default=None)
-        Customized objective function.
+        Customized objective function. Only for independent boosting.
+        The GPBoost algorithm currently does not support this.
         Should accept two parameters: preds, train_data,
         and return (grad, hess).
 
@@ -603,7 +608,7 @@ def cv(params, train_set, num_boost_round=100,
         For multi-class task, the preds is group by class_id first, then group by row_id.
         If you want to get i-th row preds in j-th class, the access way is preds[j * num_data + i].
         To ignore the default metric corresponding to the used objective,
-        set ``metrics`` to the string ``"None"``.
+        set ``metric`` to the string ``"None"``.
     init_model : string, Booster or None, optional (default=None)
         Filename of GPBoost model or Booster instance used for continue training.
     feature_name : list of strings or 'auto', optional (default="auto")
@@ -646,6 +651,8 @@ def cv(params, train_set, num_boost_round=100,
         The score of the metric is calculated again after each training step, so there is some impact on performance.
     return_cvbooster : bool, optional (default=False)
         Whether to return Booster models trained on each fold through ``CVBooster``.
+    metrics : string, list of strings or None, discontinued (default=None)
+        This is discontinued. Use the renamed equivalent argument 'metric' instead
 
     Returns
     -------
@@ -661,7 +668,7 @@ def cv(params, train_set, num_boost_round=100,
     -------
     >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
     >>> data_train = gpb.Dataset(X, y)
-    >>> params = {'objective': 'regression_l2', 'verbose': 0}
+    >>> params = {'learning_rate': 0.01, 'max_depth': 3, 'num_leaves': 2**10, 'verbose': 0}
     >>> cvbst = gpb.cv(params=params, train_set=data_train,
     >>>                gp_model=gp_model, use_gp_model_for_validation=True,
     >>>                num_boost_round=1000, early_stopping_rounds=5,
@@ -671,6 +678,9 @@ def cv(params, train_set, num_boost_round=100,
         Authors of the LightGBM Python package
         Fabio Sigrist
     """
+    if metrics is not None:
+        raise GPBoostError("The argument 'metrics' is discontinued. "
+                           "Use the renamed equivalent argument 'metric' instead")
     if fit_GP_cov_pars_OOS:
         raise ValueError("The GPBoostOOS algorithm (fit_GP_cov_pars_OOS=True) is not yet implemented in Python.")
     if not isinstance(train_set, Dataset):
@@ -717,10 +727,10 @@ def cv(params, train_set, num_boost_round=100,
     else:
         predictor = None
 
-    if metrics is not None:
+    if metric is not None:
         for metric_alias in _ConfigAliases.get("metric"):
             params.pop(metric_alias, None)
-        params['metric'] = metrics
+        params['metric'] = metric
 
     train_set._update_params(params) \
         ._set_predictor(predictor) \
@@ -839,16 +849,17 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
                                 num_boost_round=100, gp_model=None,
                                 use_gp_model_for_validation=True, train_gp_model_cov_pars=True,
                                 folds=None, nfold=5, stratified=False, shuffle=True,
-                                metrics=None, fobj=None, feval=None, init_model=None,
+                                metric=None, fobj=None, feval=None, init_model=None,
                                 feature_name='auto', categorical_feature='auto',
                                 early_stopping_rounds=None, fpreproc=None,
-                                verbose_eval=1, seed=0, callbacks=None):
+                                verbose_eval=1, seed=0, callbacks=None, metrics=None):
     """Function that allows for choosing tuning parameters from a grid in a determinstic or random way using cross validation or validation data sets.
 
     Parameters
     ----------
     param_grid : dict
         Candidate parameters defining the grid over which a search is done.
+        See https://github.com/fabsig/GPBoost/blob/master/docs/Main_parameters.rst#tuning-parameters--hyperparameters-for-the-tree-boosting-part
     train_set : Dataset
         Data to be trained on.
     params : dict, optional (default=None)
@@ -880,11 +891,13 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
         Whether to perform stratified sampling.
     shuffle : bool, optional (default=True)
         Whether to shuffle before splitting data.
-    metrics : string, list of strings or None, optional (default=None)
-        Evaluation metrics. If more than one metric is provided, only the first metric will be used to choose tuning parameters
+    metric : string, list of strings or None, optional (default=None)
+        Evaluation metric for monitoring prediction accuracy on validation data.
+        If more than one metric is provided, only the first metric will be used to choose tuning parameters.
         If not None, the metric in ``params`` will be overridden.
     fobj : callable or None, optional (default=None)
-        Customized objective function.
+        Customized objective function. Only for independent boosting.
+        The GPBoost algorithm currently does not support this.
         Should accept two parameters: preds, train_data,
         and return (grad, hess).
 
@@ -923,7 +936,7 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
         For multi-class task, the preds is group by class_id first, then group by row_id.
         If you want to get i-th row preds in j-th class, the access way is preds[j * num_data + i].
         To ignore the default metric corresponding to the used objective,
-        set ``metrics`` to the string ``"None"``.
+        set ``metric`` to the string ``"None"``.
     init_model : string, Booster or None, optional (default=None)
         Filename of GPBoost model or Booster instance used for continue training.
     feature_name : list of strings or 'auto', optional (default="auto")
@@ -958,6 +971,8 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
     callbacks : list of callables or None, optional (default=None)
         List of callback functions that are applied at each iteration.
         See Callbacks in Python API for more information.
+    metrics : string, list of strings or None, discontinued (default=None)
+        This is discontinued. Use the renamed equivalent argument 'metric' instead
 
     Returns
     -------
@@ -968,29 +983,49 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
 
     Example
     -------
-    >>> param_grid = {'learning_rate': [1,0.1,0.01], 'min_data_in_leaf': [1,10,100],
-    >>>                     'max_depth': [1,3,5,10,-1]}
+    >>> param_grid = {'learning_rate': [1,0.1,0.01], 
+    >>>   'min_data_in_leaf': [10,100,1000],
+    >>>   'max_depth': [1,2,3,5,10],
+    >>>   'lambda_l2': [0,1,10]}
+    >>> other_params = {'num_leaves': 2**10, 'verbose': 0}
+    >>> # Note: here we try different values for 'max_depth' and thus set 'num_leaves' to a large value.
+    >>> #       An alternative strategy is to impose no limit on 'max_depth',  
+    >>> #       and try different values for 'num_leaves' as follows:
+    >>> # param_grid = {'learning_rate': [1,0.1,0.01], 
+    >>> #               'min_data_in_leaf': [10,100,1000],
+    >>> #               'num_leaves': 2**np.arange(1,11),
+    >>> #               'lambda_l2': [0,1,10]}
+    >>> # other_params = {'max_depth': -1, 'verbose': 0}
     >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
     >>> data_train = gpb.Dataset(X, y)
-    >>> params = {'objective': 'regression_l2', 'verbose': 0}
-    >>> opt_params = gpb.grid_search_tune_parameters(param_grid=param_grid,
-    >>>                                              params=params,
-    >>>                                              num_try_random=None,
-    >>>                                              nfold=4,
-    >>>                                              gp_model=gp_model,
-    >>>                                              use_gp_model_for_validation=True,
-    >>>                                              train_set=data_train,
-    >>>                                              verbose_eval=1,
-    >>>                                              num_boost_round=1000,
-    >>>                                              early_stopping_rounds=10,
+    >>> opt_params = gpb.grid_search_tune_parameters(param_grid=param_grid, params=other_params,
+    >>>                                              num_try_random=None, nfold=4,
+    >>>                                              train_set=data_train, gp_model=gp_model,
+    >>>                                              use_gp_model_for_validation=True, verbose_eval=1,
+    >>>                                              num_boost_round=1000, early_stopping_rounds=10,
     >>>                                              seed=1000)
     >>> print("Best parameters: " + str(opt_params['best_params']))
     >>> print("Best number of iterations: " + str(opt_params['best_iter']))
     >>> print("Best score: " + str(opt_params['best_score']))
+    >>> 
+    >>> # Using manually defined validation data instead of cross-validation
+    >>> permute_aux = np.random.permutation(n)
+    >>> train_tune_idx = permute_aux[0:int(0.8 * n)]
+    >>> valid_tune_idx = permute_aux[int(0.8 * n):n]
+    >>> folds = [(train_tune_idx, valid_tune_idx)]
+    >>> opt_params = gpb.grid_search_tune_parameters(param_grid=param_grid, params=other_params,
+    >>>                                              num_try_random=None, folds=folds,
+    >>>                                              train_set=data_train, gp_model=gp_model,
+    >>>                                              use_gp_model_for_validation=True, verbose_eval=1,
+    >>>                                              num_boost_round=1000, early_stopping_rounds=10,
+    >>>                                              seed=1000)
 
     :Authors:
         Fabio Sigrist
     """
+    if metrics is not None:
+        raise GPBoostError("The argument 'metrics' is discontinued. "
+                           "Use the renamed equivalent argument 'metric' instead")
     # Check correct format
     if not isinstance(param_grid, dict):
         raise ValueError("param_grid needs to be a dict")
@@ -1011,10 +1046,10 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
                                                   data_name=param, check_data_type=False,
                                                   check_must_be_int=False, convert_to_type=None)
     higher_better = False
-    if metrics is not None:
-        if isinstance(metrics, str):
-            metrics = [metrics]
-        if metrics[0].startswith(('auc', 'ndcg@', 'map@', 'average_precision')):
+    if metric is not None:
+        if isinstance(metric, str):
+            metric = [metric]
+        if metric[0].startswith(('auc', 'ndcg@', 'map@', 'average_precision')):
             higher_better = True
     elif feval is not None:
         if callable(feval):
@@ -1024,15 +1059,15 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
     grid_size = _get_grid_size(param_grid)
     if num_try_random is not None:
         if num_try_random > grid_size:
-            raise ValueError("num_try_random is larger than the number of all possible combinations of parameters in param_grid")
+            raise ValueError("num_try_random is larger than the number of all possible combinations of parameters in param_grid ")
         try_param_combs = np.random.RandomState(seed).choice(a=grid_size, size=num_try_random, replace=False)
         if verbose_eval >= 1:
             print("Starting random grid search with " + str(num_try_random) + " trials out of " + str(
-                grid_size) + " parameter combinations...")
+                grid_size) + " parameter combinations ")
     else:
         try_param_combs = range(grid_size)
         if verbose_eval >= 1:
-            print("Starting deterministic grid search with " + str(grid_size) + " parameter combinations...")
+            print("Starting deterministic grid search with " + str(grid_size) + " parameter combinations ")
     if verbose_eval < 2:
         verbose_eval_cv = False
     else:
@@ -1045,31 +1080,43 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
     best_params = {}
     best_num_boost_round = num_boost_round
     counter_num_comb = 1
+    if 'max_bin' in param_grid:
+        if train_set.handle is not None:
+            raise ValueError("'train_set' cannot be constructed already when 'max_bin' is in 'param_grid' ")
+        else:
+            train_set_not_constructed = copy.deepcopy(train_set)
     for param_comb_number in try_param_combs:
         param_comb = _get_param_combination(param_comb_number=param_comb_number, param_grid=param_grid)
         for param in param_comb:
             params[param] = param_comb[param]
         if verbose_eval >= 1:
             print("Trying parameter combination " + str(counter_num_comb) +
-                  " of " + str(len(try_param_combs)) + ": " + str(param_comb) + " ...")
-        cvbst = cv(params=params, train_set=train_set, num_boost_round=num_boost_round,
-                   gp_model=gp_model, use_gp_model_for_validation=use_gp_model_for_validation,
-                   train_gp_model_cov_pars=train_gp_model_cov_pars,
-                   folds=folds, nfold=nfold, stratified=stratified, shuffle=shuffle,
-                   metrics=metrics, fobj=fobj, feval=feval, init_model=init_model,
-                   feature_name=feature_name, categorical_feature=categorical_feature,
-                   early_stopping_rounds=early_stopping_rounds, fpreproc=fpreproc,
-                   verbose_eval=verbose_eval_cv, seed=seed, callbacks=callbacks,
-                   eval_train_metric=False, return_cvbooster=False)
+                  " of " + str(len(try_param_combs)) + ": " + str(param_comb))
+        if 'max_bin' in param_grid:
+            train_set = copy.deepcopy(train_set_not_constructed)
         current_score_is_better = False
-        if higher_better:
-            current_score = np.max(cvbst[next(iter(cvbst))])
-            if current_score > best_score:
-                current_score_is_better = True
-        else:
-            current_score = np.min(cvbst[next(iter(cvbst))])
-            if current_score < best_score:
-                current_score_is_better = True
+        try:
+            cvbst = cv(params=params, train_set=train_set, num_boost_round=num_boost_round,
+                       gp_model=gp_model, use_gp_model_for_validation=use_gp_model_for_validation,
+                       train_gp_model_cov_pars=train_gp_model_cov_pars,
+                       folds=folds, nfold=nfold, stratified=stratified, shuffle=shuffle,
+                       metric=metric, fobj=fobj, feval=feval, init_model=init_model,
+                       feature_name=feature_name, categorical_feature=categorical_feature,
+                       early_stopping_rounds=early_stopping_rounds, fpreproc=fpreproc,
+                       verbose_eval=verbose_eval_cv, seed=seed, callbacks=callbacks,
+                       eval_train_metric=False, return_cvbooster=False)
+            if higher_better:
+                current_score = np.max(cvbst[next(iter(cvbst))])
+                if current_score > best_score:
+                    current_score_is_better = True
+            else:
+                current_score = np.min(cvbst[next(iter(cvbst))])
+                if current_score < best_score:
+                    current_score_is_better = True
+        except Exception as err:
+            print("Error for parameter combination " + str(counter_num_comb) +
+                  " of " + str(len(try_param_combs)) + ": " + str(param_comb) + ". Error message: ")
+            print(str(err))
         if current_score_is_better:
             best_score = current_score
             best_params = param_comb
